#include <cstdlib>
#include <iostream>
#include <memory>
#include <vector>

#include "constexp.h"
#include "ifswitch.h"
#include "initlist.h"
#include "null.h"
#include "structured_binding.h"
#include "version.h"

int main(int argc, char **argv) {

  std::cout << "Running " << PROJECT_NAME << ", version " << VERSION << "\n"
            << std::endl;

  std::vector<std::tuple<const char *, std::unique_ptr<ChapterTest>>>
      chapter_2_tests;

  chapter_2_tests.emplace_back("Nullptr test", std::make_unique<NullTest>());
  chapter_2_tests.emplace_back("If/Switch test",
                               std::make_unique<IfswitchTest>());
  chapter_2_tests.emplace_back("Constexpr test",
                               std::make_unique<ConstExpTest>());
  chapter_2_tests.emplace_back("Initializer list test",
                               std::make_unique<InitListTest>());
  chapter_2_tests.emplace_back("Structured binding test",
                               std::make_unique<StructuredBindingTest>());

  for (auto &[test_description, test] : chapter_2_tests) {
    std::cout << "\n" << test_description << ":" << std::endl;
    test->run();
  }

  std::cout << "\nExiting" << std::endl;

  return EXIT_SUCCESS;
}
